@@ -237,12 +237,12 @@ private:
     }
 
     /* Puts method as key, target as value and returns non-null (or nullptr on error). */
-    static inline char *consumeRequestLine(char *data, HttpRequest::Header &header) {
-        /* Scan until single SP, assume next is / (origin request) */
+    static inline char *consumeRequestLine(char *data, HttpRequest::Header &header, bool* isAncientHttp) {
+        /* Scan until single SP, assume next is not SP (origin request) */
         char *start = data;
         /* This catches the post padded CR and fails */
         while (data[0] > 32) data++;
-        if (data[0] == 32 && data[1] == '/') {
+        if (data[0] == 32 && data[1] != 32) {
             header.key = {start, (size_t) (data - start)};
             data++;
             /* Scan for less than 33 (catches post padded CR and fails) */
@@ -256,6 +256,12 @@ private:
                     header.value = {start, (size_t) (data - start)};
                     /* Check that the following is http 1.1 */
                     if (memcmp(" HTTP/1.1\r\n", data, 11) == 0) {
+                        *isAncientHttp = false;
+                        return data + 11;
+                    }
+                    /* Check that the following is ancient http 1.0 */
+                    if (memcmp(" HTTP/1.0\r\n", data, 11) == 0) {
+                        *isAncientHttp = true;
                         return data + 11;
                     }
                     return nullptr;
@@ -281,7 +287,7 @@ private:
     }
 
     /* End is only used for the proxy parser. The HTTP parser recognizes "\ra" as invalid "\r\n" scan and breaks. */
-    static unsigned int getHeaders(char *postPaddedBuffer, char *end, struct HttpRequest::Header *headers, void *reserved) {
+    static unsigned int getHeaders(char *postPaddedBuffer, char *end, struct HttpRequest::Header *headers, void *reserved, bool*isAncientHttp) {
         char *preliminaryKey, *preliminaryValue, *start = postPaddedBuffer;
 
         #ifdef UWS_WITH_PROXY
@@ -311,9 +317,9 @@ private:
          * which is then removed, and our counters to flip due to overflow and we end up with a crash */
 
         /* The request line is different from the field names / field values */
-        if (!(postPaddedBuffer = consumeRequestLine(postPaddedBuffer, headers[0]))) {
+        if (!(postPaddedBuffer = consumeRequestLine(postPaddedBuffer, headers[0], isAncientHttp))) {
             /* Error - invalid request line */
-            return 0;
+            return -1;
         }
         headers++;
 
@@ -399,14 +405,14 @@ private:
          * This is to always catch scan for \r but not for \r\n. */
         data[length] = '\r';
         data[length + 1] = 'a'; /* Anything that is not \n, to trigger "invalid request" */
-
-        for (unsigned int consumed; length && (consumed = getHeaders(data, data + length, req->headers, reserved)); ) {
+        bool isAncientHttp = false;
+        for (unsigned int consumed; length && (consumed = getHeaders(data, data + length, req->headers, reserved, &isAncientHttp)); ) {
             data += consumed;
             length -= consumed;
             consumedTotal += consumed;
 
             /* Store HTTP version (ancient 1.0 or 1.1) */
-            req->ancientHttp = false;
+            req->ancientHttp = isAncientHttp;
 
             /* Add all headers to bloom filter */
             req->bf.reset();
@@ -511,6 +517,8 @@ private:
                 break;
             }
         }
+        if(consumedTotal < 0)
+            return {0, FULLPTR};    
         return {consumedTotal, user};
     }
 
